@@ -12,6 +12,7 @@ use std::sync::atomic;
 use std::sync::Arc;
 use std::sync::RwLock;
 use std::time::Duration;
+use std::time::Instant;
 use submap::{BroadcastMap, SubMap};
 use tokio::io::{AsyncReadExt, AsyncWriteExt, BufReader, BufWriter};
 use tokio::net::{TcpListener, TcpStream, UnixListener, UnixStream};
@@ -30,6 +31,7 @@ use crate::OP_ACK;
 
 use crate::borrow::Cow;
 use crate::client::AsyncClient;
+use crate::common::{BrokerInfo, BrokerStats};
 #[cfg(feature = "rpc")]
 use crate::common::{ClientInfo, ClientList};
 use crate::{EventChannel, OpConfirm};
@@ -75,14 +77,21 @@ macro_rules! make_confirm_channel {
 }
 
 macro_rules! send {
-    ($db:expr, $client:expr, $target:expr, $header: expr, $buf:expr, $payload_pos:expr) => {{
+    ($db:expr, $client:expr, $target:expr, $header: expr,
+     $buf:expr, $payload_pos:expr, $len: expr) => {{
+        $client.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
+        $client.r_bytes.fetch_add($len, atomic::Ordering::SeqCst);
+        $db.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
+        $db.r_bytes.fetch_add($len, atomic::Ordering::SeqCst);
         trace!("elbus message from {} to {}", $client, $target);
         let tx = {
-            $db.clients
-                .read()
-                .unwrap()
-                .get($target)
-                .map(|c| c.tx.clone())
+            $db.clients.read().unwrap().get($target).map(|c| {
+                c.w_frames.fetch_add(1, atomic::Ordering::SeqCst);
+                c.w_bytes.fetch_add($len, atomic::Ordering::SeqCst);
+                $db.w_frames.fetch_add(1, atomic::Ordering::SeqCst);
+                $db.w_bytes.fetch_add($len, atomic::Ordering::SeqCst);
+                c.tx.clone()
+            })
         };
         if let Some(tx) = tx {
             let frame = Arc::new(FrameData {
@@ -101,8 +110,14 @@ macro_rules! send {
 }
 
 macro_rules! send_broadcast {
-    ($db:expr, $client:expr, $target:expr, $header: expr, $buf:expr, $payload_pos:expr) => {{
+    ($db:expr, $client:expr, $target:expr, $header: expr,
+     $buf:expr, $payload_pos:expr, $len: expr) => {{
+        $client.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
+        $client.r_bytes.fetch_add($len, atomic::Ordering::SeqCst);
+        $db.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
+        $db.r_bytes.fetch_add($len, atomic::Ordering::SeqCst);
         trace!("elbus broadcast message from {} to {}", $client, $target);
+        #[allow(clippy::mutable_key_type)]
         let subs = { $db.broadcasts.read().unwrap().get_clients_by_mask($target) };
         if !subs.is_empty() {
             let frame = Arc::new(FrameData {
@@ -113,7 +128,13 @@ macro_rules! send_broadcast {
                 buf: $buf,
                 payload_pos: $payload_pos,
             });
+            $db.w_frames
+                .fetch_add(subs.len() as u64, atomic::Ordering::SeqCst);
+            $db.w_bytes
+                .fetch_add($len * subs.len() as u64, atomic::Ordering::SeqCst);
             for sub in subs {
+                sub.w_frames.fetch_add(1, atomic::Ordering::SeqCst);
+                sub.w_bytes.fetch_add($len, atomic::Ordering::SeqCst);
                 let _r = sub.tx.send(frame.clone()).await;
             }
         }
@@ -121,8 +142,14 @@ macro_rules! send_broadcast {
 }
 
 macro_rules! publish {
-    ($db:expr, $client:expr, $topic:expr, $header: expr, $buf:expr, $payload_pos:expr) => {{
+    ($db:expr, $client:expr, $topic:expr, $header: expr,
+     $buf:expr, $payload_pos:expr, $len: expr) => {{
+        $client.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
+        $client.r_bytes.fetch_add($len, atomic::Ordering::SeqCst);
+        $db.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
+        $db.r_bytes.fetch_add($len, atomic::Ordering::SeqCst);
         trace!("elbus topic publish from {} to {}", $client, $topic);
+        #[allow(clippy::mutable_key_type)]
         let subs = { $db.subscriptions.read().unwrap().get_subscribers($topic) };
         if !subs.is_empty() {
             let frame = Arc::new(FrameData {
@@ -133,7 +160,13 @@ macro_rules! publish {
                 buf: $buf,
                 payload_pos: $payload_pos,
             });
+            $db.w_frames
+                .fetch_add(subs.len() as u64, atomic::Ordering::SeqCst);
+            $db.w_bytes
+                .fetch_add($len * subs.len() as u64, atomic::Ordering::SeqCst);
             for sub in subs {
+                sub.w_frames.fetch_add(1, atomic::Ordering::SeqCst);
+                sub.w_bytes.fetch_add($len, atomic::Ordering::SeqCst);
                 let _r = sub.tx.send(frame.clone()).await;
             }
         }
@@ -212,7 +245,8 @@ impl AsyncClient for Client {
         payload: Cow<'async_trait>,
         qos: QoS,
     ) -> Result<OpConfirm, Error> {
-        send!(self.db, self.client, target, None, payload.to_vec(), 0)?;
+        let len = payload.len() as u64;
+        send!(self.db, self.client, target, None, payload.to_vec(), 0, len)?;
         make_confirm_channel!(qos)
     }
     #[inline]
@@ -223,13 +257,15 @@ impl AsyncClient for Client {
         payload: Cow<'async_trait>,
         qos: QoS,
     ) -> Result<OpConfirm, Error> {
+        let len = (payload.len() + header.len()) as u64;
         send!(
             self.db,
             self.client,
             target,
             Some(header.to_vec()),
             payload.to_vec(),
-            0
+            0,
+            len
         )?;
         make_confirm_channel!(qos)
     }
@@ -240,7 +276,8 @@ impl AsyncClient for Client {
         payload: Cow<'async_trait>,
         qos: QoS,
     ) -> Result<OpConfirm, Error> {
-        send_broadcast!(self.db, self.client, target, None, payload.to_vec(), 0);
+        let len = payload.len() as u64;
+        send_broadcast!(self.db, self.client, target, None, payload.to_vec(), 0, len);
         make_confirm_channel!(qos)
     }
     #[inline]
@@ -250,7 +287,8 @@ impl AsyncClient for Client {
         payload: Cow<'async_trait>,
         qos: QoS,
     ) -> Result<OpConfirm, Error> {
-        publish!(self.db, self.client, topic, None, payload.to_vec(), 0);
+        let len = payload.len() as u64;
+        publish!(self.db, self.client, topic, None, payload.to_vec(), 0, len);
         make_confirm_channel!(qos)
     }
     #[inline]
@@ -326,6 +364,10 @@ struct ElbusClient {
     port: Option<String>,
     tx: async_channel::Sender<Frame>,
     registered: atomic::AtomicBool,
+    r_frames: atomic::AtomicU64,
+    r_bytes: atomic::AtomicU64,
+    w_frames: atomic::AtomicU64,
+    w_bytes: atomic::AtomicU64,
 }
 
 impl fmt::Display for ElbusClient {
@@ -351,6 +393,10 @@ impl ElbusClient {
                 port,
                 tx,
                 registered: atomic::AtomicBool::new(false),
+                r_frames: atomic::AtomicU64::new(0),
+                r_bytes: atomic::AtomicU64::new(0),
+                w_frames: atomic::AtomicU64::new(0),
+                w_bytes: atomic::AtomicU64::new(0),
             },
             rx,
         )
@@ -428,6 +474,11 @@ struct BrokerDb {
     subscriptions: RwLock<SubMap<BrokerClient>>,
     #[cfg(feature = "rpc")]
     rpc_client: Arc<Mutex<Option<RpcClient>>>,
+    r_frames: atomic::AtomicU64,
+    r_bytes: atomic::AtomicU64,
+    w_frames: atomic::AtomicU64,
+    w_bytes: atomic::AtomicU64,
+    startup_time: Instant,
 }
 
 impl Default for BrokerDb {
@@ -443,11 +494,25 @@ impl Default for BrokerDb {
             subscriptions: RwLock::new(SubMap::new().separator('/').match_any("+").wildcard("#")),
             #[cfg(feature = "rpc")]
             rpc_client: <_>::default(),
+            r_frames: atomic::AtomicU64::new(0),
+            r_bytes: atomic::AtomicU64::new(0),
+            w_frames: atomic::AtomicU64::new(0),
+            w_bytes: atomic::AtomicU64::new(0),
+            startup_time: Instant::now(),
         }
     }
 }
 
 impl BrokerDb {
+    fn stats(&self) -> BrokerStats {
+        BrokerStats {
+            uptime: self.startup_time.elapsed().as_secs(),
+            r_frames: self.r_frames.load(atomic::Ordering::SeqCst),
+            r_bytes: self.r_bytes.load(atomic::Ordering::SeqCst),
+            w_frames: self.w_frames.load(atomic::Ordering::SeqCst),
+            w_bytes: self.w_bytes.load(atomic::Ordering::SeqCst),
+        }
+    }
     #[cfg(feature = "rpc")]
     #[inline]
     async fn announce(&self, mut event: BrokerEvent<'_>) -> Result<(), Error> {
@@ -531,6 +596,9 @@ struct BrokerRpcHandlers {
     db: Arc<BrokerDb>,
 }
 
+#[cfg(feature = "rpc")]
+const RPC_OK: [u8; 5] = [129, 162, 111, 107, 195];
+
 #[cfg(feature = "rpc")]
 #[async_trait]
 impl RpcHandlers for BrokerRpcHandlers {
@@ -546,6 +614,24 @@ impl RpcHandlers for BrokerRpcHandlers {
             rmp_serde::from_read_ref(event.payload())?
         };
         match event.parse_method()? {
+            "test" => {
+                if !params.is_empty() {
+                    return Err(RpcError::params());
+                }
+                Ok(Some(RPC_OK.to_vec()))
+            }
+            "info" => {
+                if !params.is_empty() {
+                    return Err(RpcError::params());
+                }
+                Ok(Some(rmp_serde::to_vec_named(&Broker::info())?))
+            }
+            "stats" => {
+                if !params.is_empty() {
+                    return Err(RpcError::params());
+                }
+                Ok(Some(rmp_serde::to_vec_named(&self.db.stats())?))
+            }
             "client.list" => {
                 if !params.is_empty() {
                     return Err(RpcError::params());
@@ -559,6 +645,11 @@ impl RpcHandlers for BrokerRpcHandlers {
                         tp: v.tp.as_str(),
                         source: v.source.as_deref(),
                         port: v.port.as_deref(),
+                        r_frames: v.r_frames.load(atomic::Ordering::SeqCst),
+                        r_bytes: v.r_bytes.load(atomic::Ordering::SeqCst),
+                        w_frames: v.w_frames.load(atomic::Ordering::SeqCst),
+                        w_bytes: v.w_bytes.load(atomic::Ordering::SeqCst),
+                        queue: v.tx.len(),
                     })
                     .collect();
                 clients.sort();
@@ -660,17 +751,27 @@ where
 
 impl Default for Broker {
     fn default() -> Self {
-        Self::new()
+        Self {
+            db: <_>::default(),
+            services: <_>::default(),
+            queue_size: DEFAULT_QUEUE_SIZE,
+        }
     }
 }
 
 impl Broker {
     pub fn new() -> Self {
-        let broker_db: Arc<BrokerDb> = <_>::default();
-        Self {
-            db: broker_db,
-            services: <_>::default(),
-            queue_size: DEFAULT_QUEUE_SIZE,
+        Self::default()
+    }
+    #[inline]
+    pub fn stats(&self) -> BrokerStats {
+        self.db.stats()
+    }
+    #[inline]
+    pub fn info<'a>() -> BrokerInfo<'a> {
+        BrokerInfo {
+            author: crate::AUTHOR,
+            version: crate::VERSION,
         }
     }
     #[cfg(feature = "rpc")]
@@ -683,6 +784,7 @@ impl Broker {
         self.set_core_rpc_client(rpc_client).await;
         Ok(())
     }
+    #[inline]
     pub fn set_queue_size(&mut self, queue_size: usize) {
         self.queue_size = queue_size;
     }
@@ -1042,6 +1144,13 @@ impl Broker {
                     buf.push(OP_ACK);
                     buf.extend_from_slice(op_id);
                     buf.push($code);
+                    client.w_frames.fetch_add(1, atomic::Ordering::SeqCst);
+                    client
+                        .w_bytes
+                        .fetch_add(buf.len() as u64, atomic::Ordering::SeqCst);
+                    db.w_frames.fetch_add(1, atomic::Ordering::SeqCst);
+                    db.w_bytes
+                        .fetch_add(buf.len() as u64, atomic::Ordering::SeqCst);
                     client
                         .tx
                         .send(Arc::new(FrameData {
@@ -1057,6 +1166,13 @@ impl Broker {
             }
             match op {
                 FrameOp::SubscribeTopic => {
+                    client.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
+                    client
+                        .r_bytes
+                        .fetch_add(u64::from(len), atomic::Ordering::SeqCst);
+                    db.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
+                    db.r_bytes
+                        .fetch_add(u64::from(len), atomic::Ordering::SeqCst);
                     let sp = buf.split(|c| *c == 0);
                     {
                         let mut sdb = db.subscriptions.write().unwrap();
@@ -1071,6 +1187,13 @@ impl Broker {
                     }
                 }
                 FrameOp::UnsubscribeTopic => {
+                    client.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
+                    client
+                        .r_bytes
+                        .fetch_add(u64::from(len), atomic::Ordering::SeqCst);
+                    db.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
+                    db.r_bytes
+                        .fetch_add(u64::from(len), atomic::Ordering::SeqCst);
                     let sp = buf.split(|c| *c == 0);
                     {
                         let mut sdb = db.subscriptions.write().unwrap();
@@ -1093,7 +1216,8 @@ impl Broker {
                     drop(sp);
                     match op {
                         FrameOp::Message => {
-                            if let Err(e) = send!(db, client, target, None, buf, payload_pos) {
+                            let len = buf.len() as u64;
+                            if let Err(e) = send!(db, client, target, None, buf, payload_pos, len) {
                                 if qos == QoS::Processed {
                                     send_ack!(e.kind as u8);
                                 }
@@ -1102,13 +1226,15 @@ impl Broker {
                             }
                         }
                         FrameOp::Broadcast => {
-                            send_broadcast!(db, client, target, None, buf, payload_pos);
+                            let len = buf.len() as u64;
+                            send_broadcast!(db, client, target, None, buf, payload_pos, len);
                             if qos == QoS::Processed {
                                 send_ack!(RESPONSE_OK);
                             }
                         }
                         FrameOp::PublishTopic => {
-                            publish!(db, client, target, None, buf, payload_pos);
+                            let len = buf.len() as u64;
+                            publish!(db, client, target, None, buf, payload_pos, len);
                             if qos == QoS::Processed {
                                 send_ack!(RESPONSE_OK);
                             }
