@@ -3,7 +3,7 @@ use atty::Stream;
 use clap::{Parser, Subcommand};
 use colored::Colorize;
 use elbus::client::AsyncClient;
-use elbus::common::ClientList;
+use elbus::common::{BrokerInfo, BrokerStats, ClientList};
 use elbus::ipc::{Client, Config};
 use elbus::rpc::{DummyHandlers, Rpc, RpcClient, RpcError, RpcEvent, RpcHandlers, RpcResult};
 use elbus::{empty_payload, Error, Frame, QoS};
@@ -20,6 +20,7 @@ use tokio::time::sleep;
 #[macro_use]
 extern crate bma_benchmark;
 
+#[cfg(not(feature = "std-alloc"))]
 #[global_allocator]
 static ALLOC: jemallocator::Jemalloc = jemallocator::Jemalloc;
 
@@ -43,7 +44,13 @@ where
 #[derive(Subcommand, Clone)]
 enum BrokerCommand {
     #[clap(name = "client.list")]
-    Clients,
+    ClientList,
+    #[clap(name = "info")]
+    Info,
+    #[clap(name = "stats")]
+    Stats,
+    #[clap(name = "test")]
+    Test,
 }
 
 #[derive(Parser, Clone)]
@@ -121,8 +128,8 @@ struct Opts {
     path: String,
     #[clap(short = 'n', long = "name")]
     name: Option<String>,
-    #[clap(long = "read-buf", default_value = "8192")]
-    read_buf: usize,
+    #[clap(long = "buf-size", default_value = "8192")]
+    buf_size: usize,
     #[clap(long = "queue-size", default_value = "8192")]
     queue_size: usize,
     #[clap(long = "timeout", default_value = "5")]
@@ -198,7 +205,11 @@ async fn print_payload(payload: &[u8], silent: bool) {
         if !silent {
             println!("MSGPACK:");
         }
-        if let Ok(s) = serde_json::to_string_pretty(&map) {
+        if let Ok(s) = if silent {
+            serde_json::to_string(&map)
+        } else {
+            serde_json::to_string_pretty(&map)
+        } {
             println!("{}", s);
         } else {
             for (k, v) in map {
@@ -324,7 +335,7 @@ async fn get_payload(candidate: &Option<String>) -> Vec<u8> {
 
 async fn create_client(opts: &Opts, name: &str) -> Client {
     let config = Config::new(&opts.path, name)
-        .read_buf(opts.read_buf)
+        .read_buf(opts.buf_size)
         .queue_size(opts.queue_size)
         .timeout(Duration::from_secs_f32(opts.timeout));
     Client::connect(&config)
@@ -557,7 +568,7 @@ async fn main() {
         Command::Broker(ref op) => {
             let client = create_client(&opts, &client_name).await;
             match op {
-                BrokerCommand::Clients => {
+                BrokerCommand::ClientList => {
                     let mut rpc = RpcClient::new(client, DummyHandlers {});
                     let result = rpc
                         .call(".broker", "client.list", empty_payload!())
@@ -566,19 +577,56 @@ async fn main() {
                     let mut clients: ClientList =
                         rmp_serde::from_read_ref(result.payload()).unwrap();
                     clients.clients.sort();
-                    let mut table = ctable(vec!["name", "type", "source", "port"]);
+                    let mut table = ctable(vec![
+                        "name", "type", "source", "port", "r_frames", "r_bytes", "w_frames",
+                        "w_bytes", "queue",
+                    ]);
                     for c in clients.clients {
                         if c.name != client_name {
                             table.add_row(row![
                                 c.name,
                                 c.tp,
                                 c.source.unwrap_or_default(),
-                                c.port.unwrap_or_default()
+                                c.port.unwrap_or_default(),
+                                fnum!(c.r_frames),
+                                fnum!(c.r_bytes),
+                                fnum!(c.w_frames),
+                                fnum!(c.w_bytes),
+                                fnum!(c.queue),
                             ]);
                         }
                     }
                     table.printstd();
                 }
+                BrokerCommand::Stats => {
+                    let mut rpc = RpcClient::new(client, DummyHandlers {});
+                    let result = rpc
+                        .call(".broker", "stats", empty_payload!())
+                        .await
+                        .unwrap();
+                    let stats: BrokerStats = rmp_serde::from_read_ref(result.payload()).unwrap();
+                    let mut table = ctable(vec!["field", "value"]);
+                    table.add_row(row!["r_frames", stats.r_frames]);
+                    table.add_row(row!["r_bytes", stats.r_bytes]);
+                    table.add_row(row!["w_frames", stats.w_frames]);
+                    table.add_row(row!["w_bytes", stats.w_bytes]);
+                    table.add_row(row!["uptime", stats.uptime]);
+                    table.printstd();
+                }
+                BrokerCommand::Info => {
+                    let mut rpc = RpcClient::new(client, DummyHandlers {});
+                    let result = rpc.call(".broker", "info", empty_payload!()).await.unwrap();
+                    let info: BrokerInfo = rmp_serde::from_read_ref(result.payload()).unwrap();
+                    let mut table = ctable(vec!["field", "value"]);
+                    table.add_row(row!["author", info.author]);
+                    table.add_row(row!["version", info.version]);
+                    table.printstd();
+                }
+                BrokerCommand::Test => {
+                    let mut rpc = RpcClient::new(client, DummyHandlers {});
+                    let result = rpc.call(".broker", "test", empty_payload!()).await.unwrap();
+                    print_payload(result.payload(), opts.silent).await;
+                }
             }
         }
         Command::Listen(ref cmd) => {
