@@ -8,12 +8,17 @@ use serde_value::Value;
 use std::collections::HashMap;
 
 #[cfg_attr(feature = "rpc", derive(Serialize, Deserialize))]
-#[derive(Eq, PartialEq)]
+#[derive(Eq, PartialEq, Clone)]
 pub struct ClientInfo<'a> {
     pub name: &'a str,
     pub tp: &'a str,
     pub source: Option<&'a str>,
     pub port: Option<&'a str>,
+    pub r_frames: u64,
+    pub r_bytes: u64,
+    pub w_frames: u64,
+    pub w_bytes: u64,
+    pub queue: usize,
 }
 impl<'a> Ord for ClientInfo<'a> {
     fn cmp(&self, other: &Self) -> std::cmp::Ordering {
@@ -26,11 +31,29 @@ impl<'a> PartialOrd for ClientInfo<'a> {
     }
 }
 #[cfg_attr(feature = "rpc", derive(Serialize, Deserialize))]
+#[derive(Clone)]
 pub struct ClientList<'a> {
     #[cfg_attr(feature = "rpc", serde(borrow))]
     pub clients: Vec<ClientInfo<'a>>,
 }
 
+#[cfg_attr(feature = "rpc", derive(Serialize, Deserialize))]
+#[derive(Clone)]
+pub struct BrokerStats {
+    pub uptime: u64,
+    pub r_frames: u64,
+    pub r_bytes: u64,
+    pub w_frames: u64,
+    pub w_bytes: u64,
+}
+
+#[cfg_attr(feature = "rpc", derive(Serialize, Deserialize))]
+#[derive(Clone)]
+pub struct BrokerInfo<'a> {
+    pub author: &'a str,
+    pub version: &'a str,
+}
+
 #[allow(clippy::ptr_arg)]
 #[cfg(feature = "rpc")]
 pub fn str_to_params_map<'a>(s: &'a Vec<&str>) -> Result<HashMap<&'a str, Value>, Error> {
